#include <sstream>    /* track largest color masked object with white square and send serial coordinates/area */
#include <iostream>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/imgproc/imgproc.hpp>
#include <wiringSerial.h>

using namespace cv;
using namespace std;

int main() {
  VideoCapture cap(0);
  int fd(serialOpen("/dev/ttyAMA0", 115200));
  int clkf(getTickFrequency());
  int start, end, big, fps, x, y;
  Mat img, imgCopy, hsv, thresh, threshCopy;
  stringstream ss;
  namedWindow("A");
  namedWindow("B");
  moveWindow("A", 400, 100);
  moveWindow("B", 600, 100);

  while(true) {
    start = getTickCount();
    cap >> img;
    resize(img, img, Size(110, 80));
    img.copyTo(imgCopy);
    blur(img, img, Size(3, 3));
    cvtColor(img, hsv, CV_BGR2HSV);
    inRange(hsv, Scalar(5, 75, 75), Scalar(15, 255, 255), thresh);
    thresh.copyTo(threshCopy);
//  morphological opening (remove small objects from the foreground)
    erode(thresh, thresh, getStructuringElement(MORPH_ELLIPSE, Size(5, 5)));
    dilate(thresh, thresh, getStructuringElement(MORPH_ELLIPSE, Size(5, 5)));
//  morphological closing (fill small holes in the foreground)
    dilate(thresh, thresh, getStructuringElement(MORPH_ELLIPSE, Size(5, 5)));
    erode(thresh, thresh, getStructuringElement(MORPH_ELLIPSE, Size(5, 5)));
    vector<vector<Point> > contours;
    vector<Vec4i> hierarchy;
    findContours(thresh, contours, hierarchy, CV_RETR_TREE, CV_CHAIN_APPROX_SIMPLE, Point(0, 0));
    vector<Moments> mu(contours.size());
    int maxArea(0);
    for (unsigned int cnt(0); cnt < contours.size(); cnt++) {
      int area = contourArea(contours[cnt]);
      mu[cnt] = moments(contours[cnt], false);
      if (area > maxArea) {
        maxArea = area;
        big = cnt;
      }
    }
    if (maxArea > 0) {
      x = mu[big].m10 / mu[big].m00;
      y = mu[big].m01 / mu[big].m00;
      rectangle(imgCopy, Point(x - 5, y - 5), Point(x + 5, y + 5), Scalar(255, 255, 255), 2, 8, 0);
      ss.str("");
      ss << "FPS " << fps;
      putText(imgCopy, ss.str(), Point(0, 15), FONT_HERSHEY_COMPLEX_SMALL, 0.9, Scalar(0, 255, 255), 1, 8, 0);
      ss.str("");
      ss << "X " << x; /* setup to print x variable to video screen */
      putText(imgCopy, ss.str(), Point(0, 75), FONT_HERSHEY_COMPLEX_SMALL, 0.9, Scalar(0, 255, 255), 1, 8, 0);
      serialPuts(fd, "!&");
      serialPutchar(fd, x);
      serialPutchar(fd, y);
      serialPutchar(fd, maxArea & 0xff);
      serialPutchar(fd, maxArea >> 8);
    }
    imshow("A", imgCopy);
    imshow("B", threshCopy);
    if (waitKey(1) == 27) {   /* escape key exits program */
      serialClose(fd);
      cap.release();
      destroyAllWindows();
      break;
    }
    end = getTickCount();
    fps = clkf / (end  - start);
  }
}
